@@ -61,6 +61,11 @@ static PHP_FUNCTION(swoole_stream_socket_pair);
 static PHP_FUNCTION(swoole_user_func_handler);
 SW_EXTERN_C_END
 
+#if PHP_VERSION_ID >= 80000
+static void inherit_class(const char *child_name, size_t child_length, const char *parent_name, size_t parent_length);
+static void detach_parent_class(const char *child_name);
+static void clear_class_entries();
+#endif
 static int socket_set_option(php_stream *stream, int option, int value, void *ptrparam);
 static php_stream_size_t socket_read(php_stream *stream, char *buf, size_t count);
 static php_stream_size_t socket_write(php_stream *stream, const char *buf, size_t count);
@@ -200,6 +205,7 @@ static const zend_function_entry swoole_sockets_functions[] = {
 // clang-format on
 
 static zend_array *tmp_function_table = nullptr;
+static std::unordered_map<std::string, zend_class_entry *> child_class_entries;
 
 SW_EXTERN_C_BEGIN
 #include "ext/standard/file.h"
@@ -277,6 +283,10 @@ void php_swoole_runtime_rshutdown() {
     zend_hash_destroy(tmp_function_table);
     efree(tmp_function_table);
     tmp_function_table = nullptr;
+
+#if PHP_VERSION_ID >= 80000
+    clear_class_entries();
+#endif
 }
 
 void php_swoole_runtime_mshutdown() {
@@ -1420,6 +1430,10 @@ bool PHPCoroutine::enable_hook(uint32_t flags) {
             SW_HOOK_SOCKETS_FUNC(socket_close);
             SW_HOOK_SOCKETS_FUNC(socket_clear_error);
             SW_HOOK_SOCKETS_FUNC(socket_last_error);
+
+#if PHP_VERSION_ID >= 80000
+            inherit_class(ZEND_STRL("Swoole\\Coroutine\\Socket"), ZEND_STRL("Socket"));
+#endif
         }
     } else {
         if (runtime_hook_flags & PHPCoroutine::HOOK_BLOCKING_FUNCTION) {
@@ -1448,6 +1462,10 @@ bool PHPCoroutine::enable_hook(uint32_t flags) {
             SW_UNHOOK_FUNC(socket_close);
             SW_UNHOOK_FUNC(socket_clear_error);
             SW_UNHOOK_FUNC(socket_last_error);
+
+#if PHP_VERSION_ID >= 80000
+            detach_parent_class("Swoole\\Coroutine\\Socket");
+#endif
         }
     }
 
@@ -1813,6 +1831,18 @@ static PHP_FUNCTION(swoole_stream_select) {
     Z_PARAM_LONG(usec)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
+    double timeout = -1;
+    if (!secnull) {
+        if (sec < 0) {
+            php_error_docref(nullptr, E_WARNING, "The seconds parameter must be greater than 0");
+            RETURN_FALSE;
+        } else if (usec < 0) {
+            php_error_docref(nullptr, E_WARNING, "The microseconds parameter must be greater than 0");
+            RETURN_FALSE;
+        }
+        timeout = (double) sec + ((double) usec / 1000000);
+    }
+
     std::unordered_map<int, PollSocket> fds;
 
     if (r_array != nullptr) {
@@ -1832,17 +1862,12 @@ static PHP_FUNCTION(swoole_stream_select) {
         RETURN_FALSE;
     }
 
-    double timeout = -1;
-    if (!secnull) {
-        if (sec < 0) {
-            php_error_docref(nullptr, E_WARNING, "The seconds parameter must be greater than 0");
-            RETURN_FALSE;
-        } else if (usec < 0) {
-            php_error_docref(nullptr, E_WARNING, "The microseconds parameter must be greater than 0");
-            RETURN_FALSE;
+    ON_SCOPE_EXIT {
+        for (auto &i : fds) {
+            zend::KeyValue *kv = (zend::KeyValue *) i.second.ptr;
+            delete kv;
         }
-        timeout = (double) sec + ((double) usec / 1000000);
-    }
+    };
 
     /* slight hack to support buffered data; if there is data sitting in the
      * read buffer of any of the streams in the read array, let's pretend
@@ -1893,7 +1918,6 @@ static PHP_FUNCTION(swoole_stream_select) {
             }
             retval++;
         }
-        delete kv;
     }
 
     RETURN_LONG(retval);
@@ -2044,3 +2068,62 @@ static PHP_FUNCTION(swoole_user_func_handler) {
     real_func *rf = (real_func *) zend_hash_find_ptr(tmp_function_table, execute_data->func->common.function_name);
     zend_call_function(&fci, rf->fci_cache);
 }
+
+#if PHP_VERSION_ID >= 80000
+zend_class_entry *find_class_entry(const char *name, size_t length) {
+    zend_string *search_key = zend_string_init(name, length, 0);
+    zend_class_entry *class_ce = zend_lookup_class(search_key);
+    zend_string_release(search_key);
+    return class_ce ? class_ce : nullptr;
+}
+
+static void inherit_class(const char *child_name, size_t child_length, const char *parent_name, size_t parent_length) {
+    zend_class_entry *temp_ce = nullptr;
+    zend_class_entry *child_ce = find_class_entry(child_name, child_length);
+    zend_class_entry *parent_ce = find_class_entry(parent_name, parent_length);
+
+    if (!child_ce || !parent_ce || instanceof_function(child_ce, parent_ce)) {
+        return;
+    }
+
+    temp_ce = child_ce;
+    while (temp_ce->parent) {
+        temp_ce = temp_ce->parent;
+    }
+    temp_ce->parent = parent_ce;
+
+    std::string key(ZSTR_VAL(child_ce->name));
+    child_class_entries.insert({key, child_ce});
+}
+
+void start_detach_parent_class(zend_class_entry *class_ce) {
+    zend_class_entry *p1 = nullptr;
+    zend_class_entry *p2 = nullptr;
+
+    p1 = class_ce;
+    p2 = class_ce->parent;
+    while (p2->parent) {
+        p1 = p1->parent;
+        p2 = p2->parent;
+    }
+
+    p1->parent = nullptr;
+}
+
+static void detach_parent_class(const char *child_name) {
+    std::string search_key(child_name);
+    auto iter = child_class_entries.find(search_key);
+    if (iter == child_class_entries.end()) {
+        return;
+    }
+    start_detach_parent_class(iter->second);
+    child_class_entries.erase(search_key);
+}
+
+static void clear_class_entries() {
+    for (auto iter = child_class_entries.begin(); iter != child_class_entries.end(); iter++) {
+        start_detach_parent_class(iter->second);
+    }
+    child_class_entries.clear();
+}
+#endif
