@@ -10,7 +10,7 @@
   | to obtain it through the world-wide-web, please send a note to       |
   | license@swoole.com so we can mail you a copy immediately.            |
   +----------------------------------------------------------------------+
-  | Author: Tianfeng Han  <mikan.tenny@gmail.com>                        |
+  | Author: Tianfeng Han  <rango@swoole.com>                             |
   +----------------------------------------------------------------------+
 */
 #ifndef SWOOLE_CONFIG_H_
@@ -208,8 +208,6 @@
 #define SW_HTTP_SERVER_SOFTWARE "swoole-http-server"
 #define SW_HTTP_PARAM_MAX_NUM 128
 #define SW_HTTP_FORM_KEYLEN 512
-#define SW_HTTP_COOKIE_KEYLEN 128
-#define SW_HTTP_COOKIE_VALLEN 4096
 #define SW_HTTP_RESPONSE_INIT_SIZE 65536
 #define SW_HTTP_HEADER_MAX_SIZE 65536
 #define SW_HTTP_HEADER_KEY_SIZE 128
@@ -225,8 +223,20 @@
 #define SW_HTTP_BAD_REQUEST_PACKET "HTTP/1.1 400 Bad Request\r\n\r\n"
 #define SW_HTTP_REQUEST_ENTITY_TOO_LARGE_PACKET "HTTP/1.1 413 Request Entity Too Large\r\n\r\n"
 #define SW_HTTP_SERVICE_UNAVAILABLE_PACKET "HTTP/1.1 503 Service Unavailable\r\n\r\n"
-#define SW_HTTP_PAGE_400 "<html><body><h2>HTTP 400 Bad Request</h2><hr><i>Powered by Swoole</i></body></html>"
-#define SW_HTTP_PAGE_404 "<html><body><h2>HTTP 404 Not Found</h2><hr><i>Powered by Swoole</i></body></html>"
+
+#define SW_HTTP_PAGE_CSS "<style> \
+body { padding: 0.5em; line-height: 2; } \
+h1 { font-size: 1.5em; padding-bottom: 0.3em; border-bottom: 1px solid #ccc; } \
+ul { list-style-type: disc; } \
+footer { border-top: 1px solid #ccc; } \
+a { color: #0969da; } \
+</style>"
+
+#define SW_HTTP_POWER_BY  "<footer><i>Powered by Swoole</i></footer>"
+
+#define SW_HTTP_PAGE_400 "<html><body>" SW_HTTP_PAGE_CSS "<h1>HTTP 400 Bad Request</h1>" SW_HTTP_POWER_BY "</body></html>"
+#define SW_HTTP_PAGE_404 "<html><body>" SW_HTTP_PAGE_CSS "<h1>HTTP 404 Not Found</h1>" SW_HTTP_POWER_BY "</body></html>"
+#define SW_HTTP_PAGE_500 "<html><body>" SW_HTTP_PAGE_CSS "<h1>HTTP 500 Internal Server Error</h1>" SW_HTTP_POWER_BY "</body></html>"
 
 /**
  * HTTP2 Protocol
@@ -272,6 +282,7 @@
 #define SW_DEFAULT_C_STACK_SIZE (2 * 1024 * 1024)
 #define SW_CORO_SUPPORT_BAILOUT 1
 #define SW_CORO_SWAP_BAILOUT 1
+#define SW_CORO_BAILOUT_EXIT_CODE 1
 //#define SW_CONTEXT_PROTECT_STACK_PAGE    1
 //#define SW_CONTEXT_DETECT_STACK_USAGE    1
 
