@@ -10,7 +10,7 @@
   | to obtain it through the world-wide-web, please send a note to       |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
-  | Author: Tianfeng Han  <mikan.tenny@gmail.com>                        |
+  | Author: Tianfeng Han  <rango@swoole.com>                             |
   +----------------------------------------------------------------------+
 */
 
@@ -658,8 +658,8 @@ class Server {
         DISPATCH_UIDMOD = 5,
         DISPATCH_USERFUNC = 6,
         DISPATCH_STREAM = 7,
-        DISPATCH_CO_CONN_LB,
-        DISPATCH_CO_REQ_LB,
+        DISPATCH_CO_CONN_LB = 8,
+        DISPATCH_CO_REQ_LB = 9,
     };
 
     enum FactoryDispatchResult {
