@@ -10,7 +10,7 @@
   | to obtain it through the world-wide-web, please send a note to       |
   | license@swoole.com so we can mail you a copy immediately.            |
   +----------------------------------------------------------------------+
-  | Author: Tianfeng Han  <mikan.tenny@gmail.com>                        |
+  | Author: Tianfeng Han  <rango@swoole.com>                             |
   |         Twosee  <twose@qq.com>                                       |
   +----------------------------------------------------------------------+
 */
@@ -20,10 +20,10 @@
 
 #define SWOOLE_MAJOR_VERSION 4
 #define SWOOLE_MINOR_VERSION 8
-#define SWOOLE_RELEASE_VERSION 8
+#define SWOOLE_RELEASE_VERSION 9
 #define SWOOLE_EXTRA_VERSION ""
-#define SWOOLE_VERSION "4.8.8"
-#define SWOOLE_VERSION_ID 40808
+#define SWOOLE_VERSION "4.8.9"
+#define SWOOLE_VERSION_ID 40809
 #define SWOOLE_API_VERSION_ID 0x202109a
 
 #define SWOOLE_BUG_REPORT                                                                                              \
